// flip.cpp : Implementation of main().

#include "flip-curses.h"
#include "flip-std.h"

#include <unistd.h>

#include <iostream>
#include <cstdlib>

int main(int argc, char* argv[])
{
    char mode = ' ';
    int c, result = -1;
    long speed = 50;
    const char* infile = NULL;
    const char* outfile = NULL;
    
#ifdef FLIP_CURSES_SUPPORT
# define OPT_C "c"
#else
# define OPT_C
#endif
    while ((c = getopt(argc, argv, OPT_C "hi:o:s:")) != -1)
    {
        switch (c)
        {
#ifdef FLIP_CURSES_SUPPORT
        case 'c':
            mode = 'c';
            break;
#endif
        case 'h':
            if (result < 0)
                result = 0;
            break;
        case 'i':
            infile = optarg;
            break;
        case 'o':
            outfile = optarg;
            break;
        case 's':
            speed = std::atol(optarg);
            break;
        case '?':
            result = 1;
            break;
        }
    }
    if (result < 0 && optind > argc - 1)
    {
        std::cerr << argv[0] << ": Not enough arguments" << std::endl;
        result = 1;
    }
    else if (result < 0 && optind < argc - 1)
    {
        std::cerr << argv[0] << ": Too many arguments" << std::endl;
        result = 1;
    }
    if (result >= 0)
    {
        std::cout << argv[0] << " - interpreter for the Flip programming language.\n"
            "Usage:\n"
            "    " << argv[0] << " [-c] [-i file] [-o file] [-s delay] source-file\n"
            "    " << argv[0] << " -h\n"
            "Options:\n"
#ifdef FLIP_CURSES_SUPPORT
            "  -c    Show the program excecution in the console.\n"
#else
            " (-c)   This build does not support the -c option.\n"
#endif
            "  -h    Show this help and exit.\n"
            "  -i file\n"
            "        Read input from the specified file.\n"
            "  -o file\n"
            "        Write output to the specified file.\n"
            "  -s delay\n"
            "        Set the excecution speed (only affects -c)." << std::endl;
        return result;
    }
    char* sourcefile = argv[optind++];
    
    switch (mode)
    {
    case ' ':
        result = flip_std(sourcefile, infile, outfile);
        break;
#ifdef FLIP_CURSES_SUPPORT
    case 'c':
        result = flip_curses(sourcefile, infile, outfile, speed);
        break;
#endif
    }
    
    return result;
}
